"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var webhooks_1 = __importDefault(require("@octokit/webhooks"));
var bunyan_1 = __importDefault(require("bunyan"));
exports.Logger = bunyan_1.default;
var cache_manager_1 = __importDefault(require("cache-manager"));
var application_1 = require("./application");
exports.Application = application_1.Application;
var context_1 = require("./context");
exports.Context = context_1.Context;
var github_app_1 = require("./github-app");
var logger_1 = require("./logger");
var resolver_1 = require("./resolver");
var server_1 = require("./server");
var webhook_proxy_1 = require("./webhook-proxy");
// tslint:disable:no-var-requires
// These needs types
var logRequestErrors = require('./middleware/log-request-errors');
var cache = cache_manager_1.default.caching({
    store: 'memory',
    ttl: 60 * 60 // 1 hour
});
var defaultAppFns = [
    require('./apps/default'),
    require('./apps/sentry'),
    require('./apps/stats')
];
// tslint:enable:no-var-requires
var Probot = /** @class */ (function () {
    function Probot(options) {
        var _this = this;
        options.webhookPath = options.webhookPath || '/';
        options.secret = options.secret || 'development';
        this.options = options;
        this.logger = logger_1.logger;
        this.apps = [];
        this.webhook = new webhooks_1.default({ path: options.webhookPath, secret: options.secret });
        this.app = github_app_1.createApp({ id: options.id, cert: options.cert });
        this.server = server_1.createServer({ webhook: this.webhook.middleware, logger: logger_1.logger });
        // Log all received webhooks
        this.webhook.on('*', function (event) {
            return _this.receive(event);
        });
        // Log all webhook errors
        this.webhook.on('error', this.errorHandler);
    }
    Probot.prototype.errorHandler = function (err) {
        var errMessage = err.message.toLowerCase();
        if (errMessage.includes('x-hub-signature')) {
            logger_1.logger.error({ err: err }, 'Go to https://github.com/settings/apps/YOUR_APP and verify that the Webhook secret matches the value of the WEBHOOK_SECRET environment variable.');
        }
        else if (errMessage.includes('pem') || errMessage.includes('json web token')) {
            logger_1.logger.error({ err: err }, 'Your private key (usually a .pem file) is not correct. Go to https://github.com/settings/apps/YOUR_APP and generate a new PEM file. If you\'re deploying to Now, visit https://probot.github.io/docs/deployment/#now.');
        }
        else {
            logger_1.logger.error(err);
        }
    };
    Probot.prototype.receive = function (event) {
        this.logger.debug({ event: event }, 'Webhook received');
        return Promise.all(this.apps.map(function (app) { return app.receive(event); }));
    };
    Probot.prototype.load = function (appFn) {
        if (typeof appFn === 'string') {
            appFn = resolver_1.resolve(appFn);
        }
        var app = new application_1.Application({ app: this.app, cache: cache, catchErrors: true });
        // Connect the router from the app to the server
        this.server.use(app.router);
        // Initialize the ApplicationFunction
        app.load(appFn);
        this.apps.push(app);
        return app;
    };
    Probot.prototype.setup = function (appFns) {
        var _this = this;
        // Log all unhandled rejections
        process.on('unhandledRejection', this.errorHandler);
        // Load the given appFns along with the default ones
        appFns.concat(defaultAppFns).forEach(function (appFn) { return _this.load(appFn); });
        // Register error handler as the last middleware
        this.server.use(logRequestErrors);
    };
    Probot.prototype.start = function () {
        if (this.options.webhookProxy) {
            webhook_proxy_1.createWebhookProxy({
                logger: logger_1.logger,
                path: this.options.webhookPath,
                port: this.options.port,
                url: this.options.webhookProxy
            });
        }
        this.server.listen(this.options.port);
        logger_1.logger.info('Listening on http://localhost:' + this.options.port);
    };
    return Probot;
}());
exports.Probot = Probot;
exports.createProbot = function (options) { return new Probot(options); };
//# sourceMappingURL=index.js.map